package be.iminds.ilabt.jfed.experimenter_gui.dialogs;

import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.events.OpenESpecEvent;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.UIUtils;
import be.iminds.ilabt.jfed.gui_preferences.HLPreferenceKey;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.authority.finder.AuthorityFinder;
import be.iminds.ilabt.jfed.preferences.JFedPreferences;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.GuiceFXMLUtil;
import com.google.common.eventbus.EventBus;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.control.Button;
import javafx.scene.control.ButtonBar;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Dialog;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.net.MalformedURLException;

/**
 * User: twalcari
 * Date: 2/24/14
 * Time: 4:03 PM
 */
public class OpenESpecDialog extends Dialog<ButtonType> {
    private static final Logger LOG = LoggerFactory.getLogger(OpenESpecDialog.class);

    @Nonnull private final OpenESpecController controller;
    @Nonnull private final JFedPreferences jFedPreferences;
    @Nonnull private final AuthorityFinder authorityFinder;
    @Nonnull private final GeniUserProvider geniUserProvider;
    @Nonnull private final EventBus eventBus;

    // if not null, instead of a new experiment with the chosen ESpec,
    // the ESpec is loaded and started in the existing experiment
    @Nullable private ExperimentController experimentController;

    @Inject
    public OpenESpecDialog(@Nonnull GuiceFXMLUtil guiceFXMLUtil,
                           @Nonnull JFedPreferences jFedPreferences,
                           @Nonnull AuthorityFinder authorityFinder,
                           @Nonnull GeniUserProvider geniUserProvider,
                           @Nonnull EventBus eventBus) {
        this.jFedPreferences = jFedPreferences;
        this.authorityFinder = authorityFinder;
        this.geniUserProvider = geniUserProvider;
        this.eventBus = eventBus;

        GuiceFXMLUtil.Result<?, OpenESpecController> result =
                guiceFXMLUtil.createFromFXML(OpenESpecController.class);

        this.controller = result.getController();

        setTitle("Start Experiment using ESpec");
        setGraphic(GlyphUtils.createDialogGlyph(FontAwesome.Glyph.FLASK, Color.CHOCOLATE));
        setHeaderText("Choose an ESpec");
        //TODO: is manual decoration pane wrapping still needed?
        getDialogPane().setContent(UIUtils.wrapWithDecorationPane(result.getRoot()));
        setResizable(true);

        ButtonType openESpecAction =
                new ButtonType("Start ESpec", ButtonBar.ButtonData.OK_DONE); //this::checkAndOpenESpec);
        getDialogPane().getButtonTypes().setAll(openESpecAction, ButtonType.CANCEL);

        Button openESpecButton = (Button) getDialogPane().lookupButton(openESpecAction);
        openESpecButton.disableProperty().bind(controller.validProperty().not());

        EventHandler<ActionEvent> openESpecEventHandler = event -> {
                    if (controller.isValid())
                    openESpec();
                    event.consume();
                };
        openESpecButton.addEventFilter(ActionEvent.ACTION, openESpecEventHandler);
        controller.setOnAction(openESpecEventHandler);
    }



    /**
     * This method uses the inputs from the dialog to start the experiment.
     * These inputs should be validated before calling this function!
     */
    private void openESpec() {
        hide();

        try {
            OpenESpecEvent openESpecEvent = new OpenESpecEvent(experimentController, controller.getESpecSourceString());
            LOG.info("User pressed \"Open Espec\" button: {}", openESpecEvent);
            eventBus.post(openESpecEvent);
            jFedPreferences.setString(HLPreferenceKey.PREF_LAST_OPENED_ESPEC, controller.getESpecSourceString());
        } catch (MalformedURLException e) {
            LOG.error("User pressed \"Open Espec\" button, but caused MalformedURLException.", e);
        }
    }

    public void setExperimentController(ExperimentController experimentController) {
        this.experimentController = experimentController;
    }
}
